#ifndef __EXPR_2OP_hDUD2
#define __EXPR_2OP_hDUD2

#include "dudloadroutines.h"

#ifndef MAKEDEPEND
#include <cmath>
#endif

#ifdef EXPR2OP_IMPLEMENT_FUNCTIONS
#define declare_expr_fun(name, doing) expr_floattype name(expr_floattype a, expr_floattype b) { return doing; }
#else
#define declare_expr_fun(name, doing) extern expr_floattype name(expr_floattype a, expr_floattype b);	
#endif

declare_expr_fun(expr_plus_fun,    a + b)
declare_expr_fun(expr_minus_fun,   a - b)
declare_expr_fun(expr_times_fun,   a * b)
declare_expr_fun(expr_divided_fun, a / b)

declare_expr_fun(expr_lt_fun,   a <  b)
declare_expr_fun(expr_gt_fun,   a >  b)
declare_expr_fun(expr_eq_fun,   a == b)
declare_expr_fun(expr_lteq_fun, a <= b)
declare_expr_fun(expr_gteq_fun, a >= b)
declare_expr_fun(expr_neq_fun,  a != b)

#undef declare_expr_fun

class expr_2op : public expr
{
public:
	exprp left;
	exprp right;
	expr_floattype (*fun)(expr_floattype,expr_floattype);
	
private:
	bool CanUseStrings;
	
public:
	expr_2op(const expr_2op &s) : expr(s)
	{
		left = s.left;
		right= s.right;
	}
	expr_2op(exprp l, expr_floattype (*f)(expr_floattype,expr_floattype)) : left(l), fun(f)
	{
		ID = ext_pairfun;		
	}
	expr_2op(exprp l, exprp r, expr_floattype (*f)(expr_floattype,expr_floattype)) : left(l), right(r), fun(f)
	{
		ID = ext_pairfun;		
	}	
	expr_2op(exprp l, string &input, int prio, expr_floattype (*f)(expr_floattype,expr_floattype)) : left(l), fun(f)
	{
		ID = ext_pairfun;
		
		CanUseStrings = (prio==prio_equality
		              || prio==prio_relational
		              || f==expr_plus_fun);
		
		right = expr::load(input, prio)->optimize();
		blank(input);
		if(!right)
			valid = false;
		else if(!CanUseStrings && (left->type==str || right->type==str))
			valid = false;
		else
			gettype();
	}
	void gettype(exprp l=NULL, exprp r=NULL)
	{
		if(!l)l=left; if(!r)r=right;
		// String comparison also returns num.
		type = ((l->type==num && r->type==num)
		     || (l->type==str && r->type==str)) ? num : notsure;
		isconst = type==num && l->isconst && r->isconst;
	}
	
	virtual exprp optimize();
	virtual const string getname() const
	{
		string op;
		if(fun==expr_plus_fun)op="+";
		else if(fun==expr_minus_fun)op="-";
		else if(fun==expr_times_fun)op="*";
		else if(fun==expr_divided_fun)op="/";
		else if(fun==expr_fmod_fun)op="%";
		else if(fun==expr_pow_fun)op="^";
		else if(fun==expr_lt_fun)op="<";
		else if(fun==expr_gt_fun)op=">";
		else if(fun==expr_eq_fun)op="==";
		else if(fun==expr_neq_fun)op="!=";
		else if(fun==expr_lteq_fun)op="<=";
		else if(fun==expr_gteq_fun)op=">=";
		else op="#";

		return '(' + left->getname() + ' ' + op + ' ' + right->getname() + ')';
	}
	virtual exprp calculate(calculate_args) const
	{
		exprp res = new expr_2op(
			left->calculate(calculate_call),
			right->calculate(calculate_call),
			fun);
		return res->optimize();
	}
};

#endif
