#ifndef __EXPR_VAR_H_dud2
#define __EXPR_VAR_H_dud2

#include "expr.h"

class expr_container : public expr
{
protected:
	string content;
public:
	expr_container(const string &s) : content(s) { }
};

class expr_number : public expr
{
	expr_floattype luku;
public:
	expr_number(expr_floattype d) : luku(d)
	{
		type    = num;
		isconst = true;
		ID = ext_number;
	}
	expr_floattype &number() { return luku; }
	virtual const string getname() const;
	virtual const string getstr() const { return getname(); }
	virtual const expr_floattype getnum() const { return luku; }
	virtual exprp calculate(calculate_args_notused) const
	{
		return new expr_number(*this);
	}
	virtual const bool truth() const
	{
		return luku==0.0 ? false : true;
	}
};

class expr_fraction : public expr
{
	expr_floattype a,b;
public:
	expr_fraction(expr_floattype d, expr_floattype e) : a(d), b(e)
	{
		type    = num;
		isconst = true;
	}
	virtual const string getname() const;
	virtual const string getstr() const { return getname(); }
	virtual const expr_floattype getnum() const { return a/b; }
	virtual exprp optimize()
	{
		if(a<0 && b<0)a=-a, b=-b;
		return this;
	}
	virtual exprp calculate(calculate_args_notused) const
	{
		return new expr_fraction(*this);
	}
	virtual const bool truth() const
	{
		return getnum()==0.0 ? false : true;
		
	//	it can be true: 0/n is 0.
	}
};

class expr_truth : public expr
{
	bool value;
public:
	expr_truth(bool b) : value(b)
	{
		type    = num;
		isconst = true;
	}
	virtual const string getstr() const;
	virtual const expr_floattype getnum() const { return value; }
	virtual exprp calculate(calculate_args_notused) const
	{
		return new expr_truth(*this);
	}
	virtual const bool truth() const
	{
		return value;
	}
};

class expr_string : public expr_container
{
public:
	expr_string(const string &s) : expr_container(s)
	{
//		cout << "Made string '" << s << "'\n";
		type    = str;
		isconst = true;
	}
	virtual const string getstr() const
	{
//		cout << "<Asked string '" << content << "'>\n";
		return content;
	}
	virtual const string getname() const
	{
		return '"' + content + '"';
	}
	virtual exprp calculate(calculate_args) const
	{
		return new expr_string(*this);
	}
};

class expr_variable : public expr_container
{
public:
	expr_variable(const string &s) : expr_container(s)
	{
		type    = notsure;
		isconst = true;
	}
	virtual const string getname() const
	{
		return content;
	}
	virtual const string getstr() const
	{
		return "";
	}
	#ifndef DUD2
	virtual exprp calculate(calculate_args) const
	{
		return new expr_variable(*this);
	}
	#else
	virtual exprp calculate(calculate_args) const;
	virtual obj::vartype &getvar(calculate_args) const;
	#endif
};

#endif
